import imageio
import typing as ty
import numpy  # noqa: F401
import typing  # noqa: F401
from fileformats.core import FileSet, extra_implementation
from fileformats.image.raster import RasterImage, DataArrayType


@extra_implementation(FileSet.load)
def read_raster_data(image: RasterImage, **kwargs: ty.Any) -> DataArrayType:
    return imageio.imread(image.fspath)  # type: ignore


@extra_implementation(FileSet.save)
def write_raster_data(
    image: RasterImage, data: DataArrayType, **kwargs: ty.Any
) -> None:
    imageio.imwrite(image.fspath, data, **kwargs)



# import sys
# import pytest
from fileformats.application import Json, Yaml


SAMPLE_JSON = (
    """{"a": "string field", "alist": [0, 1, 2, 3, 4, 5], """
    """"anesteddict": {"x": null, "y": [], "z": 42.0}}"""
)

SAMPLE_YAML = """a: string field
alist:
- 0
- 1
- 2
- 3
- 4
- 5
anesteddict:
  x: null
  y: []
  z: 42.0
"""


# @pytest.mark.xfail(
#     sys.version_info.minor <= 9,
#     reason="upstream Pydra issue with type-checking 'type' objects",
# )
def test_json_to_yaml(work_dir):
    in_file = work_dir / "test.json"
    with open(in_file, "w") as f:
        f.write(SAMPLE_JSON)
    jsn = Json(in_file)
    yml = Yaml.convert(jsn)
    assert yml.raw_contents == SAMPLE_YAML


# @pytest.mark.xfail(
#     sys.version_info.minor <= 9,
#     reason="upstream Pydra issue with type-checking 'type' objects",
# )
def test_yaml_to_json(work_dir):
    in_file = work_dir / "test.yaml"
    with open(in_file, "w") as f:
        f.write(SAMPLE_JSON)
    yml = Yaml(in_file)
    jsn = Json.convert(yml)
    assert jsn.raw_contents == SAMPLE_JSON



from pathlib import Path
import typing as ty
import tempfile
from pydra.design import python
from fileformats.core import converter
from fileformats.image.raster import RasterImage, Bitmap, Gif, Jpeg, Png, Tiff


@converter(target_format=Bitmap, output_format=Bitmap)  # type: ignore[misc]
@converter(target_format=Gif, output_format=Gif)  # type: ignore[misc]
@converter(target_format=Jpeg, output_format=Jpeg)  # type: ignore[misc]
@converter(target_format=Png, output_format=Png)  # type: ignore[misc]
@converter(target_format=Tiff, output_format=Tiff)  # type: ignore[misc]
@python.define(outputs={"out_file": RasterImage})  # type: ignore[misc]
def convert_image(
    in_file: RasterImage,
    output_format: ty.Type[RasterImage],
    out_dir: ty.Optional[Path] = None,
) -> RasterImage:
    data_array = in_file.load()
    if out_dir is None:
        out_dir = Path(tempfile.mkdtemp())
    output_path = out_dir / (
        in_file.fspath.stem + (output_format.ext if output_format.ext else "")
    )
    return output_format.new(output_path, data_array)



import os.path
import sys
import typing as ty
import tempfile
import tarfile
import zipfile
from pathlib import Path
from pydra.design import python
from fileformats.generic import FsObject
from fileformats.core.utils import set_cwd
from fileformats.core.typing import PathType
from fileformats.core import converter, FileSet
from fileformats.application import Zip, Tar, TarGzip


TAR_COMPRESSION_TYPES = ["", "gz", "bz2", "xz"]

TAR_COMPRESSION_ANNOT = (
    str,
    {
        "help": (
            "The type of compression applied to tar file, "
            "', '".join(TAR_COMPRESSION_TYPES)
        ),
        "allowed_values": list(TAR_COMPRESSION_TYPES),
    },
)

ZIP_COMPRESSION_ANNOT = (
    int,
    {
        "help": (
            "The type of compression applied to zip file, "
            "see https://docs.python.org/3/library/zipfile.html#zipfile.ZIP_DEFLATED "
            "for valid compression types"
        ),
        "allowed_values": [
            zipfile.ZIP_STORED,
            zipfile.ZIP_DEFLATED,
            zipfile.ZIP_BZIP2,
            zipfile.ZIP_LZMA,
        ],
    },
)

Compressed = FileSet.type_var("Compressed")

FilterMethodType = ty.Any
# FIXME: This is a placeholder for the actual type, when pydra supports it properly
# FilterMethodType = ty.Optional[
#     ty.Callable[[tarfile.TarInfo], ty.Optional[tarfile.TarInfo]]
# ]


@converter(source_format=FsObject, target_format=Tar)  # type: ignore[misc]
@converter(source_format=FsObject, target_format=TarGzip, compression="gz")  # type: ignore[misc]
@converter(source_format=Compressed, target_format=Tar[Compressed])  # type: ignore[misc]
@converter(
    source_format=Compressed, target_format=TarGzip[Compressed], compression="gz"  # type: ignore[misc]
)
@python.define(outputs={"out_file": Path})  # type: ignore[misc]
def create_tar(
    in_file: FsObject,
    out_file: ty.Optional[Path] = None,
    base_dir: ty.Optional[Path] = None,
    filter: FilterMethodType = None,
    compression: ty.Optional[str] = None,
    format: int = tarfile.DEFAULT_FORMAT,
    ignore_zeros: bool = False,
    encoding: str = tarfile.ENCODING,
) -> Path:

    if len(in_file.fspaths) > 1:
        raise NotImplementedError(
            "Can only archive file-sets with single paths currently"
        )

    ext: str
    if not compression:
        compression = ""
        ext = ".tar"
    else:
        ext = ".tar." + compression

    if not out_file:
        out_file = Path(Path(in_file).name + ext)

    if base_dir is None:
        base_dir = Path(in_file).parent

    out_file = out_file.absolute()

    with tarfile.open(  # type: ignore[call-overload]
        name=out_file,
        mode=f"w:{compression}",
        format=format,
        ignore_zeros=ignore_zeros,
        encoding=encoding,
    ) as tfile, set_cwd(base_dir):
        for fspath in in_file.fspaths:
            tfile.add(relative_path(fspath, base_dir), filter=filter)

    return Path(out_file)


@converter(source_format=Tar, target_format=FsObject)  # type: ignore[misc]
@converter(source_format=TarGzip, target_format=FsObject)  # type: ignore[misc]
@converter(source_format=Tar[Compressed], target_format=Compressed)  # type: ignore[misc]
@converter(source_format=TarGzip[Compressed], target_format=Compressed)  # type: ignore[misc]
@python.define(outputs={"out_file": Path})  # type: ignore[misc]
def extract_tar(
    in_file: FsObject,
    extract_dir: ty.Optional[Path] = None,
    bufsize: int = 10240,
    compression_type: str = "*",
) -> Path:

    if extract_dir is None:
        extract_dir = Path(tempfile.mkdtemp())
    else:
        extract_dir = extract_dir.absolute()
        os.makedirs(extract_dir, exist_ok=True)
    extract_dir = Path(extract_dir)

    if not compression_type:
        compression_type = ""

    with tarfile.open(name=in_file, mode=f"r:{compression_type}") as tfile:  # type: ignore[call-overload]
        tfile.extractall(path=extract_dir)

    extracted = [extract_dir / f for f in os.listdir(extract_dir)]
    if len(extracted) > 1:
        raise NotImplementedError(
            "Can't handle zip files with more than one path currently"
        )
    return extracted[0]


@converter(source_format=FsObject, target_format=Zip)  # type: ignore[misc]
@converter(source_format=Compressed, target_format=Zip[Compressed])  # type: ignore[misc]
@python.define(outputs={"out_file": Zip})  # type: ignore[misc]
def create_zip(
    in_file: FsObject,
    out_file: ty.Optional[Path] = None,
    base_dir: ty.Optional[Path] = None,
    compression: int = zipfile.ZIP_DEFLATED,
    allowZip64: bool = True,
    compresslevel: ty.Optional[int] = None,
    strict_timestamps: bool = True,
) -> Zip:

    if len(in_file.fspaths) > 1:
        raise NotImplementedError(
            "Can only archive file-sets with single paths currently"
        )

    if out_file is None:  # type: ignore[comparison-overlap]
        out_file = Path(Path(in_file).name + ".zip")

    if base_dir is None:  # type: ignore[comparison-overlap]
        base_dir = Path(in_file).parent

    out_file = out_file.absolute()

    zip_kwargs = {}
    if not strict_timestamps:  # Truthy is the default in earlier versions
        if sys.version_info.major <= 3 and sys.version_info.minor < 8:
            raise Exception(
                "Must be using Python >= 3.8 to pass "
                f"strict_timestamps={strict_timestamps!r}"
            )

        zip_kwargs["strict_timestamps"] = strict_timestamps

    with zipfile.ZipFile(
        out_file,
        mode="w",
        compression=compression,
        allowZip64=allowZip64,
        compresslevel=compresslevel,
        **zip_kwargs,
    ) as zfile, set_cwd(base_dir):
        for fspath in in_file.fspaths:
            fspath = Path(fspath)
            if fspath.is_dir():
                for dpath, _, files in os.walk(fspath):
                    zfile.write(relative_path(dpath, base_dir))
                    for fname in files:
                        fpath = os.path.join(dpath, fname)
                        zfile.write(relative_path(fpath, base_dir))
            else:
                zfile.write(relative_path(fspath, base_dir))
    return Zip(out_file)


@converter(source_format=Zip, target_format=FsObject)  # type: ignore[misc]
@converter(source_format=Zip[Compressed], target_format=Compressed)  # type: ignore[misc]
@python.define(outputs={"out_file": Path})  # type: ignore[misc]
def extract_zip(in_file: Zip, extract_dir: ty.Optional[Path] = None) -> Path:

    if extract_dir is None:
        extract_dir = Path(tempfile.mkdtemp())
    else:
        extract_dir = extract_dir.absolute()
        os.makedirs(extract_dir, exist_ok=True)
    extract_dir = Path(extract_dir)

    with zipfile.ZipFile(in_file) as zfile:
        zfile.extractall(path=extract_dir)

    extracted = [extract_dir / f for f in os.listdir(extract_dir)]
    if len(extracted) > 1:
        raise NotImplementedError(
            "Can't handle zip files with more than one path currently"
        )
    return extracted[0]


def relative_path(path: PathType, base_dir: PathType) -> str:
    path = os.path.abspath(path)
    relpath = os.path.relpath(path, base_dir)
    if ".." in relpath:
        raise RuntimeError(
            f"Cannot add {path} to archive as it is not a "
            f"subdirectory of {base_dir}"
        )
    return relpath



import tempfile
from pathlib import Path
import typing as ty
from pydra.design import python
from fileformats.core import converter, FileSet
from fileformats.generic import DirectoryOf, SetOf, TypedDirectory, TypedSet

T = FileSet.type_var("T")


@converter(target_format=SetOf[T], source_format=DirectoryOf[T])  # type: ignore[misc]
@python.define(outputs={"out_file": TypedSet})  # type: ignore[misc]
def list_dir_contents(in_file: TypedDirectory) -> TypedSet:
    classified_set: ty.Type[TypedSet] = SetOf.__class_getitem__(*in_file.content_types)  # type: ignore[assignment, arg-type]
    return classified_set(in_file.contents)


@converter(target_format=DirectoryOf[T], source_format=SetOf[T])  # type: ignore[misc]
@python.define(outputs={"out_file": TypedDirectory})  # type: ignore[misc]
def put_contents_in_dir(
    in_file: TypedSet,
    out_dir: ty.Optional[Path] = None,
    copy_mode: FileSet.CopyMode = FileSet.CopyMode.copy,
) -> TypedDirectory:
    if out_dir is None:
        out_dir = Path(tempfile.mkdtemp())
    for fset in in_file.contents:
        fset.copy(out_dir, mode=copy_mode)
    classified_dir: ty.Type[TypedDirectory] = DirectoryOf.__class_getitem__(  # type: ignore[assignment]
        *in_file.content_types  # type: ignore[arg-type]
    )
    return classified_dir(out_dir)



import sys
import typing as ty
from pathlib import Path
import pydicom.tag
from fileformats.core import FileSet, extra_implementation
from fileformats.application import Dicom
import medimages4tests.dummy.dicom.mri.t1w.siemens.skyra.syngo_d13c
from fileformats.core import SampleFileGenerator

if sys.version_info <= (3, 11):
    from typing_extensions import TypeAlias
else:
    from typing import TypeAlias

TagListType: TypeAlias = ty.Union[
    ty.List[int],
    ty.List[str],
    ty.List[ty.Tuple[int, int]],
    ty.List[pydicom.tag.BaseTag],
]


@extra_implementation(FileSet.read_metadata)
def dicom_read_metadata(
    dicom: Dicom,
    metadata_keys: ty.Optional[TagListType] = None,
    **kwargs: ty.Any,
) -> ty.Mapping[str, ty.Any]:
    dcm = pydicom.dcmread(dicom.fspath, specific_tags=metadata_keys)
    return Dicom.pydicom_to_dict(dcm)


@extra_implementation(FileSet.generate_sample_data)
def dicom_generate_sample_data(
    dicom: Dicom,
    generator: SampleFileGenerator,
) -> ty.List[Path]:
    return next(
        medimages4tests.dummy.dicom.mri.t1w.siemens.skyra.syngo_d13c.get_image(
            out_dir=generator.dest_dir
        ).iterdir()
    )


@extra_implementation(FileSet.load)
def dicom_load(
    dicom: Dicom,
    specific_tags: ty.Optional[TagListType] = None,
    **kwargs: ty.Any,
) -> pydicom.FileDataset:
    return pydicom.dcmread(dicom.fspath, specific_tags=specific_tags)


@extra_implementation(FileSet.save)
def dicom_save(
    dicom: Dicom,
    data: pydicom.FileDataset,
    write_like_original: bool = False,
    **kwargs: ty.Any,
) -> None:
    pydicom.dcmwrite(dicom.fspath, data, write_like_original=write_like_original)



import typing as ty
from fileformats.core import extra_implementation, FileSet
from fileformats.text import Plain  # , Csv, Tsv

# import pandas as pd


@extra_implementation(FileSet.load)
def load_text_file(text: Plain, **kwargs: ty.Any) -> Plain:
    return text.raw_contents  # type: ignore[no-any-return]


@extra_implementation(FileSet.save)
def save_text_file(text: Plain, data: ty.Any, **kwargs: ty.Any) -> None:
    text.fspath.write_text(data)


# @extra_implementation(FileSet.load)
# def load_csv_file(csv_file: Csv, **kwargs: ty.Any) -> pd.DataFrame:
#     return pd.read_csv(csv_file.fspath, **kwargs)


# @extra_implementation(FileSet.save)
# def save_csv_file(csv_file: Csv, data: pd.DataFrame, **kwargs: ty.Any) -> None:
#     data.to_csv(csv_file.fspath, index=False, **kwargs)


# @extra_implementation(FileSet.load)
# def load_tsv_file(tsv_file: Tsv, **kwargs: ty.Any) -> pd.DataFrame:
#     return pd.read_csv(tsv_file.fspath, sep="\t", **kwargs)


# @extra_implementation(FileSet.save)
# def save_tsv_file(tsv_file: Tsv, data: pd.DataFrame, **kwargs: ty.Any) -> None:
#     data.to_csv(tsv_file.fspath, sep="\t", index=False, **kwargs)



import pytest
from fileformats.application import Dicom


def test_dicom_metadata():

    dicom = Dicom.sample()

    assert dicom.metadata["EchoTime"] == 2.07


def test_dicom_metadata_with_specific_tags():

    dicom = Dicom(Dicom.sample(), metadata_keys=["EchoTime"])

    assert dicom.metadata["EchoTime"] == 2.07
    with pytest.raises(KeyError):
        dicom.metadata["PatientName"]



import pytest
from pathlib import Path
import typing as ty
from fileformats.generic import DirectoryOf, SetOf
from fileformats.text import TextFile


@pytest.fixture
def file_names() -> ty.List[str]:
    return ["file1.txt", "file2.txt", "file3.txt"]


@pytest.fixture
def files_dir(file_names: ty.List[str], tmp_path: Path) -> Path:
    out_dir = tmp_path / "out"
    out_dir.mkdir()
    for fname in file_names:
        (out_dir / fname).write_text(fname)
    return out_dir


@pytest.fixture
def test_files(files_dir: Path, file_names: ty.List[str]) -> ty.List[Path]:
    return [files_dir / n for n in file_names]


def test_list_dir_contents(files_dir: Path, test_files: ty.List[Path]) -> None:
    text_set = SetOf[TextFile].convert(DirectoryOf[TextFile](files_dir))  # type: ignore[misc]
    assert sorted(t.name for t in text_set.contents) == sorted(
        f.name for f in test_files
    )


def test_put_contents_in_dir(
    file_names: ty.List[str], test_files: ty.List[Path]
) -> None:
    text_dir = DirectoryOf[TextFile].convert(SetOf[TextFile](test_files))  # type: ignore[misc]
    assert sorted(t.name for t in text_dir.contents) == file_names
